"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pattern = exports.excludes = exports.within = exports.isBoolean = exports.maxLength = exports.minLength = exports.equalsTo = exports.isEmail = exports.length = exports.isArray = exports.isObject = exports.isFunction = exports.isString = exports.isNumber = exports.required = exports.avalidate = exports.validate = exports.avalidateSingle = exports.validateSingle = exports.SkipValidation = void 0;

const SkipValidation = function (message) {
  this.name = "SkipValidation";
  this.message = message;
};

exports.SkipValidation = SkipValidation;

const formatError = (error, data, key) => {
  return error.replace("{value}", data).replace("{key}", key);
};

const validateSingle = (data, validators, multipleErrors, all, key) => {
  const errors = [];

  if (typeof validators === "function") {
    validators = [validators];
  }

  for (let i = 0; i < validators.length; i++) {
    try {
      let error = validators[i](data, all);

      if (typeof error === "string") {
        const formattedError = formatError(error, data, key);
        if (!multipleErrors) return formattedError;
        errors.push(formattedError);
      }
    } catch (err) {
      if (err instanceof SkipValidation) {
        break;
      } else {
        throw err;
      }
    }
  }

  if (errors.length > 0) return errors;
  return null;
};

exports.validateSingle = validateSingle;

const avalidateSingle = (data, validators, multipleErrors, all, key) => {
  if (typeof validators === "function") {
    validators = [validators];
  }

  async function v(funcs, errors) {
    if (errors.length > 0) {
      if (!multipleErrors) return errors[0];
    }

    if (funcs.length === 0) {
      return errors.length > 0 ? errors : null;
    }

    try {
      const currentValidator = funcs.shift();
      const error = await currentValidator(data, all);

      if (error) {
        errors.push(formatError(error, data, key));
      }

      return v(funcs, errors);
    } catch (err) {
      if (err instanceof SkipValidation) {
        if (errors.length > 0) {
          return multipleErrors ? errors : errors[0];
        }

        return null;
      } else {
        throw err;
      }
    }
  }

  return v([...validators], []);
};

exports.avalidateSingle = avalidateSingle;

function getInvalidKeys({
  dataKeys,
  validKeys
}) {
  return dataKeys.filter(key => !validKeys.includes(key));
}

const defaultOptions = {
  multipleErrors: false,
  partial: false,
  strict: true,
  invalidKeyError: 'This field is not allowed.'
};

const _validate = (data, schema, options, v) => {
  data = data || {};
  if (!schema) throw new Error("'schema' is required.");
  options = { ...defaultOptions,
    ...options
  };
  const {
    multipleErrors,
    partial,
    strict,
    invalidKeyError
  } = options;
  const dataKeys = Object.keys(data);
  let validKeys = Object.keys(schema);

  if (partial) {
    validKeys = validKeys.filter(key => dataKeys.includes(key));
  }

  if (strict) {
    const invalidKeys = getInvalidKeys({
      dataKeys,
      validKeys
    });

    if (invalidKeys.length > 0) {
      return invalidKeys.reduce((a, key) => {
        a[key] = invalidKeyError;
        return a;
      }, {});
    }
  }

  return v(validKeys, null, data, schema, multipleErrors);
};

const validate = (data, schema, options = {}) => {
  function v(keys, errors, data, schema, multipleErrors) {
    if (keys.length === 0) {
      return errors;
    }

    const key = keys.shift();
    const error = validateSingle(data[key], schema[key], multipleErrors, data, key);

    if (error) {
      errors = errors || {};
      errors[key] = error;
    }

    return v(keys, errors, data, schema, multipleErrors);
  }

  return _validate(data, schema, options, v);
};

exports.validate = validate;

const avalidate = (data, schema, options = {}) => {
  async function v(keys, errors, data, schema, multipleErrors) {
    if (keys.length === 0) {
      return errors;
    }

    const key = keys.shift();
    const error = await avalidateSingle(data[key], schema[key], multipleErrors, data, key);

    if (error) {
      errors = errors || {};
      errors[key] = error;
    }

    return v(keys, errors, data, schema, multipleErrors);
  }

  return _validate(data, schema, options, v);
};

exports.avalidate = avalidate;

const required = (flag, error) => {
  function isNullLike(value) {
    return [undefined, "", null, NaN].includes(value);
  }

  return value => {
    if (flag && isNullLike(value)) {
      return error || "This field is required.";
    } else if (!flag && isNullLike(value)) {
      // skip rest of the validators
      throw new SkipValidation();
    }
  };
};

exports.required = required;

const isNumber = error => {
  return value => {
    if (typeof value !== "number" || isNaN(value)) {
      return error || "'{value}' is not a valid number.";
    }
  };
};

exports.isNumber = isNumber;

const isString = error => {
  return value => {
    if (typeof value !== "string") {
      return error || "'{value}' is not a valid string.";
    }
  };
};

exports.isString = isString;

const isFunction = error => {
  return value => {
    if (typeof value !== "function") {
      return error || "Expected a function.";
    }
  };
};

exports.isFunction = isFunction;

const isObject = error => {
  return value => {
    if (value !== Object(value)) {
      return error || "Expected an object.";
    }
  };
};

exports.isObject = isObject;

const isArray = error => {
  return value => {
    if (Object.prototype.toString.call(value) !== "[object Array]") {
      return error || "Expected an array.";
    }
  };
};

exports.isArray = isArray;

const length = (length, error) => {
  return value => {
    let str = value + "";

    if (str.length !== length) {
      return error || `It must be ${length} characters long.`;
    }
  };
};

exports.length = length;

const isEmail = error => {
  return value => {
    let pattern = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;

    if (!pattern.test(value)) {
      return error || "Invalid email id.";
    }
  };
};

exports.isEmail = isEmail;

const equalsTo = (key, error) => {
  return (value, all) => {
    if (value !== all[key]) {
      return error || `'{key}' and '${key}' do not match.`;
    }
  };
};

exports.equalsTo = equalsTo;

const minLength = (length, error) => {
  return value => {
    let str = value + "";

    if (str.length < length) {
      return error || `It must be at least ${length} characters long.`;
    }
  };
};

exports.minLength = minLength;

const maxLength = (length, error) => {
  return value => {
    let str = value + "";

    if (str.length > length) {
      return error || `It must be at most ${length} characters long.`;
    }
  };
};

exports.maxLength = maxLength;

const isBoolean = error => {
  return value => {
    if (value !== true && value !== false) {
      return error || "Invalid boolean value.";
    }
  };
};

exports.isBoolean = isBoolean;

const within = (list, error) => {
  return value => {
    if (!(value instanceof Array)) {
      value = [value];
    }

    let odds = [];

    for (let index = 0; index < value.length; index++) {
      if (list.indexOf(value[index]) === -1) {
        odds.push(value[index]);
      }
    }

    if (odds.length > 0) {
      return error || `[${odds}] do not fall under the allowed list.`;
    }
  };
};

exports.within = within;

const excludes = (list, error) => {
  return value => {
    if (!(value instanceof Array)) {
      value = [value];
    }

    let odds = [];

    for (let index = 0; index < value.length; index++) {
      if (list.indexOf(value[index]) !== -1) {
        odds.push(value[index]);
      }
    }

    if (odds.length > 0) {
      return error || `[${odds}] fall under restricted values.`;
    }
  };
};

exports.excludes = excludes;

const pattern = (regex, error) => {
  return value => {
    if (!regex.test(value)) {
      return error || "'{value}' does not match with the pattern.";
    }
  };
};

exports.pattern = pattern;